@@ -164,22 +164,28 @@ class ActiveRecord::ConnectionAdapters::NullDBAdapter < ActiveRecord::Connection
   end
 
   def exec_query(statement, name = 'SQL', binds = [], options = {})
+    internal_exec_query(statement, name, binds, **options)
+  end
+
+  def internal_exec_query(statement, name = 'SQL', binds = [], prepare: false, async: false)
     self.execution_log << Statement.new(entry_point, statement)
     EmptyResult.new
   end
 
-  def select_rows(statement, name = nil, binds = [])
+  def select_rows(statement, name = nil, binds = [], async: false)
     [].tap do
       self.execution_log << Statement.new(entry_point, statement)
     end
   end
 
-  def insert(statement, name = nil, primary_key = nil, object_id = nil, sequence_name = nil, binds = [])
-    (object_id || next_unique_id).tap do
-      with_entry_point(:insert) do
-        super(statement, name, primary_key, object_id, sequence_name)
-      end
+  def insert(statement, name = nil, primary_key = nil, object_id = nil, sequence_name = nil, binds = [], returning: nil)
+    with_entry_point(:insert) do
+      super(statement, name, primary_key, object_id, sequence_name)
     end
+
+    result = object_id || next_unique_id
+
+    returning ? [result] : result
   end
   alias :create :insert
 
@@ -191,7 +197,7 @@ class ActiveRecord::ConnectionAdapters::NullDBAdapter < ActiveRecord::Connection
 
   def delete(statement, name=nil, binds = [])
     with_entry_point(:delete) do
-      super(statement, name).size
+      super(statement, name)
     end
   end
 
@@ -262,9 +268,39 @@ class ActiveRecord::ConnectionAdapters::NullDBAdapter < ActiveRecord::Connection
     end
   end
 
+  ### Rails 8.0+ ###
+  def perform_query(raw_connection, statement, binds, type_casted_binds, prepare:, notification_payload:, batch:)
+    self.execution_log << Statement.new(entry_point, statement)
+    NullObject.new
+  end
+
+  def affected_rows(raw_result)
+    0
+  end
+
+  def valid_column_definition_options
+    super + [:array, :using, :cast_as, :as, :type, :enum_type, :stored, :srid]
+  end
+
+  READ_QUERY = ActiveRecord::ConnectionAdapters::AbstractAdapter.build_read_query_regexp(
+    :close, :declare, :fetch, :move, :set, :show
+  ) # :nodoc:
+  private_constant :READ_QUERY
+
+  def write_query?(sql) # :nodoc:
+    !READ_QUERY.match?(sql)
+  rescue ArgumentError # Invalid encoding
+    !READ_QUERY.match?(sql.b)
+  end
+
+  def reconnect
+    true
+  end
+  ### Rails 8.0+ ###
+
   protected
 
-  def select(statement, name = nil, binds = [], prepare: nil, async: nil)
+  def select(statement, name = nil, binds = [], prepare: nil, async: nil, allow_retry: nil)
     EmptyResult.new.tap do |r|
       r.bind_column_meta(columns_for(name))
       self.execution_log << Statement.new(entry_point, statement)
@@ -313,7 +349,7 @@ class ActiveRecord::ConnectionAdapters::NullDBAdapter < ActiveRecord::Connection
 
   def new_table_definition(adapter = nil, table_name = nil, is_temporary = nil, options = {})
     case ::ActiveRecord::VERSION::MAJOR
-    when 6, 7
+    when 6, 7, 8
       TableDefinition.new(self, table_name, temporary: is_temporary, options: options.except(:id))
     when 5
       TableDefinition.new(table_name, is_temporary, options.except(:id), nil)
@@ -343,13 +379,37 @@ class ActiveRecord::ConnectionAdapters::NullDBAdapter < ActiveRecord::Connection
     [nil, @logger, @config]
   end
 
+  # Register types only once to avoid ActiveRecord::TypeConflictError
+  # in ActiveRecord::Type::Registration#<=>
+  REGISTRATION_MUTEX = Mutex.new
+
   def register_types
-    require 'active_model/type'
+    REGISTRATION_MUTEX.synchronize do
+      return if self.class.types_registered
+
+      self.class.types_registered = true
+    end
+
     ActiveRecord::Type.register(
       :primary_key,
       ActiveModel::Type::Integer,
       adapter: adapter_name,
       override: true
     )
+
+    ActiveRecord::Type.add_modifier({ array: true }, DummyOID, adapter: :nulldb)
+    ActiveRecord::Type.add_modifier({ range: true }, DummyOID, adapter: :nulldb)
+  end
+
+  class << self
+    attr_accessor :types_registered
+  end
+
+  class DummyOID < ActiveModel::Type::Value
+    attr_reader :subtype
+
+    def initialize(*args)
+      @subtype = args.first
+    end
   end
 end
