@@ -3,9 +3,6 @@ require 'active_support/deprecation'
 require 'active_record/connection_adapters/nulldb_adapter'
 
 module NullDB
-  LEGACY_ACTIVERECORD = 
-    Gem::Version.new(ActiveRecord::VERSION::STRING) < Gem::Version.new('4.2.0')
-
   class Configuration < Struct.new(:project_root); end
 
   class << self
